@@ -66,7 +66,7 @@ from .utils import (
     decode_utf8,
 )
 
-from .errors import HTTPrettyError
+from .errors import HTTPrettyError, UnmockedError
 
 from datetime import datetime
 from datetime import timedelta
@@ -100,8 +100,10 @@ except ImportError:  # pragma: no cover
     ssl = None
 
 
-POTENTIAL_HTTP_PORTS = set([80, 443])
-DEFAULT_HTTP_PORTS = tuple(POTENTIAL_HTTP_PORTS)
+DEFAULT_HTTP_PORTS = frozenset([80])
+POTENTIAL_HTTP_PORTS = set(DEFAULT_HTTP_PORTS)
+DEFAULT_HTTPS_PORTS = frozenset([443])
+POTENTIAL_HTTPS_PORTS = set(DEFAULT_HTTPS_PORTS)
 
 
 class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
@@ -158,7 +160,8 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
         self.error_message = None
 
         # Parse the request based on the attributes above
-        self.parse_request()
+        if not self.parse_request():
+            return
 
         # making the HTTP method string available as the command
         self.method = self.command
@@ -170,6 +173,7 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
             self.path = self.path.encode('iso-8859-1')
         except UnicodeDecodeError:
             pass
+
         self.path = decode_utf8(self.path)
 
         qstring = self.path.split("?", 1)[-1]
@@ -208,8 +212,8 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
         content_type = self.headers.get('content-type', '')
 
         do_parse = PARSING_FUNCTIONS.get(content_type, FALLBACK_FUNCTION)
-        body = decode_utf8(body)
         try:
+            body = decode_utf8(body)
             return do_parse(body)
         except:
             return body
@@ -225,7 +229,9 @@ class HTTPrettyRequestEmpty(object):
 
 
 class FakeSockFile(StringIO):
-    pass
+    def close(self):
+        self.socket.close()
+        StringIO.close(self)
 
 
 class FakeSSLSocket(object):
@@ -244,14 +250,16 @@ class fakesock(object):
 
         def __init__(self, family=socket.AF_INET, type=socket.SOCK_STREAM,
                      protocol=0):
-            self.setsockopt(family, type, protocol)
-            self.truesock = old_socket(family, type, protocol)
+            self.truesock = (old_socket(family, type, protocol)
+                             if httpretty.allow_net_connect
+                             else None)
             self._closed = True
             self.fd = FakeSockFile()
+            self.fd.socket = self
             self.timeout = socket._GLOBAL_DEFAULT_TIMEOUT
             self._sock = self
             self.is_http = False
-            self._bufsize = 16
+            self._bufsize = 1024
 
         def getpeercert(self, *a, **kw):
             now = datetime.now()
@@ -280,22 +288,34 @@ class fakesock(object):
         def ssl(self, sock, *args, **kw):
             return sock
 
-        def setsockopt(self, family, type, protocol):
-            self.family = family
-            self.protocol = protocol
-            self.type = type
+        def setsockopt(self, level, optname, value):
+            if self.truesock:
+                self.truesock.setsockopt(level, optname, value)
 
         def connect(self, address):
-            self._address = (self._host, self._port) = address
             self._closed = False
-            self.is_http = self._port in POTENTIAL_HTTP_PORTS
+
+            try:
+                self._address = (self._host, self._port) = address
+            except ValueError:
+                # We get here when the address is just a string pointing to a
+                # unix socket path/file
+                #
+                # See issue #206
+                self.is_http = False
+            else:
+                self.is_http = self._port in POTENTIAL_HTTP_PORTS | POTENTIAL_HTTPS_PORTS
 
             if not self.is_http:
-                self.truesock.connect(self._address)
+                if self.truesock:
+                    self.truesock.connect(self._address)
+                else:
+                    raise UnmockedError()
 
         def close(self):
-            if not (self.is_http and self._closed):
-                self.truesock.close()
+            if not self.is_http and not self._closed:
+                if self.truesock:
+                    self.truesock.close()
             self._closed = True
 
         def makefile(self, mode='r', bufsize=-1):
@@ -322,12 +342,16 @@ class fakesock(object):
             buffer so that HTTPretty can return it accordingly when
             necessary.
             """
-            if self.is_http:  # no need to connect if `self.is_http` is
-                              # False because self.connect already did
-                              # that
-                self.truesock.connect(self._address)
 
-            self.truesock.settimeout(0)
+            if not self.truesock:
+                raise UnmockedError()
+
+            if not self.is_http:
+                return self.truesock.sendall(data, *args, **kw)
+
+            self.truesock.connect(self._address)
+
+            self.truesock.setblocking(1)
             self.truesock.sendall(data, *args, **kw)
 
             should_continue = True
@@ -335,7 +359,7 @@ class fakesock(object):
                 try:
                     received = self.truesock.recv(self._bufsize)
                     self.fd.write(received)
-                    should_continue = len(received) > 0
+                    should_continue = len(received) == self._bufsize
 
                 except socket.error as e:
                     if e.errno == EAGAIN:
@@ -346,7 +370,8 @@ class fakesock(object):
 
         def sendall(self, data, *args, **kw):
             self._sent_data.append(data)
-
+            self.fd = FakeSockFile()
+            self.fd.socket = self
             try:
                 requestline, _ = data.split(b'\r\n', 1)
                 method, path, version = parse_requestline(decode_utf8(requestline))
@@ -395,7 +420,7 @@ class fakesock(object):
 
             self._entry = matcher.get_next_entry(method, info, request)
 
-        def debug(self, func, *a, **kw):
+        def debug(self, truesock_func, *a, **kw):
             if self.is_http:
                 frame = inspect.stack()[0][0]
                 lines = list(map(utf8, traceback.format_stack(frame)))
@@ -408,30 +433,34 @@ class fakesock(object):
                     "".join(decode_utf8(lines)),
                 ]
                 raise RuntimeError("\n".join(message))
-            return func(*a, **kw)
+            if not self.truesock:
+                raise UnmockedError()
+            return getattr(self.truesock, truesock_func)(*a, **kw)
 
         def settimeout(self, new_timeout):
             self.timeout = new_timeout
 
         def send(self, *args, **kwargs):
-            return self.debug(self.truesock.send, *args, **kwargs)
+            return self.debug('send', *args, **kwargs)
 
         def sendto(self, *args, **kwargs):
-            return self.debug(self.truesock.sendto, *args, **kwargs)
+            return self.debug('sendto', *args, **kwargs)
 
         def recvfrom_into(self, *args, **kwargs):
-            return self.debug(self.truesock.recvfrom_into, *args, **kwargs)
+            return self.debug('recvfrom_into', *args, **kwargs)
 
         def recv_into(self, *args, **kwargs):
-            return self.debug(self.truesock.recv_into, *args, **kwargs)
+            return self.debug('recv_into', *args, **kwargs)
 
         def recvfrom(self, *args, **kwargs):
-            return self.debug(self.truesock.recvfrom, *args, **kwargs)
+            return self.debug('recvfrom', *args, **kwargs)
 
         def recv(self, *args, **kwargs):
-            return self.debug(self.truesock.recv, *args, **kwargs)
+            return self.debug('recv', *args, **kwargs)
 
         def __getattr__(self, name):
+            if not self.truesock:
+                raise UnmockedError()
             return getattr(self.truesock, name)
 
 
@@ -635,7 +664,12 @@ class URIInfo(BaseClass):
         self.port = port or 80
         self.path = path or ''
         self.query = query or ''
-        self.scheme = scheme or (self.port == 443 and "https" or "http")
+        if scheme:
+            self.scheme = scheme
+        elif self.port in POTENTIAL_HTTPS_PORTS:
+            self.scheme = 'https'
+        else:
+            self.scheme = 'http'
         self.fragment = fragment or ''
         self.last_request = last_request
 
@@ -687,7 +721,8 @@ class URIInfo(BaseClass):
 
     def get_full_domain(self):
         hostname = decode_utf8(self.hostname)
-        if self.port not in DEFAULT_HTTP_PORTS:
+        # Port 80/443 should not be appended to the url
+        if self.port not in DEFAULT_HTTP_PORTS | DEFAULT_HTTPS_PORTS:
             return ":".join([hostname, str(self.port)])
 
         return hostname
@@ -695,7 +730,10 @@ class URIInfo(BaseClass):
     @classmethod
     def from_uri(cls, uri, entry):
         result = urlsplit(uri)
-        POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
+        if result.scheme == 'https':
+            POTENTIAL_HTTPS_PORTS.add(int(result.port or 443))
+        else:
+            POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
         return cls(result.username,
                    result.password,
                    result.hostname,
@@ -715,6 +753,11 @@ class URIMatcher(object):
         self._match_querystring = match_querystring
         if type(uri).__name__ == 'SRE_Pattern':
             self.regex = uri
+            result = urlsplit(uri.pattern)
+            if result.scheme == 'https':
+                POTENTIAL_HTTPS_PORTS.add(int(result.port or 443))
+            else:
+                POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
         else:
             self.info = URIInfo.from_uri(uri, entries)
 
@@ -781,6 +824,7 @@ class httpretty(HttpBaseClass):
 
     last_request = HTTPrettyRequestEmpty()
     _is_enabled = False
+    allow_net_connect = True
 
     @classmethod
     def match_uriinfo(cls, info):
@@ -848,8 +892,8 @@ class httpretty(HttpBaseClass):
 
     @classmethod
     def reset(cls):
-        global POTENTIAL_HTTP_PORTS
-        POTENTIAL_HTTP_PORTS = set([80, 443])
+        POTENTIAL_HTTP_PORTS.intersection_update(DEFAULT_HTTP_PORTS)
+        POTENTIAL_HTTPS_PORTS.intersection_update(DEFAULT_HTTPS_PORTS)
         cls._entries.clear()
         cls.latest_requests = []
         cls.last_request = HTTPrettyRequestEmpty()
